<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:template="http://www.boddie.org.uk/ns/xmltools/template"
                version="1.0">

  <xsl:param name="types"/>

  <!-- Add or process type elements inside item elements. -->

  <xsl:template match="item">
    <!-- Copy the element and its contents. -->
    <xsl:copy>
      <xsl:apply-templates select="@*"/>
      <!-- Look for a type element. -->
      <xsl:call-template name="type"/>
      <xsl:apply-templates select="*[local-name() != 'type']"/>
    </xsl:copy>
  </xsl:template>

  <!-- Investigate type elements. -->

  <xsl:template name="type">
    <!-- Make the element. -->
    <type>
      <!-- Look for type element values. -->
      <xsl:call-template name="type-enum">
        <!-- Supply the existing type-enum values. -->
        <xsl:with-param name="values" select="type/type-enum/@value"/>
      </xsl:call-template>
    </type>
  </xsl:template>

  <!-- Add to type elements the range of acceptable values. -->

  <xsl:template name="type-enum">
    <xsl:param name="values"/>
    <!-- Find all values in the range and add them. -->
    <xsl:for-each select="$types/types/type-enum">
      <xsl:copy>
        <xsl:apply-templates select="@*"/>
        <!-- Mark those whose value are the same as that found on the element. -->
        <xsl:if test="$values[string() = current()/@value]">
          <xsl:attribute name="value-is-set">true</xsl:attribute>
        </xsl:if>
      </xsl:copy>
    </xsl:for-each>
  </xsl:template>

  <!-- Replicate unknown elements. -->

  <xsl:template match="@*|*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|*|node()"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
